from fastapi import APIRouter, HTTPException

from src.errors import Response
from src.types.characters import Character
from src.utils import load_index, parse_query_langs, parse_result_lang, load_file

characters = APIRouter(prefix='/characters', tags=['Characters'])


@characters.get('/')
async def query_characters(query_field: str = 'names', query_languages: str = 'eng') -> Response:
    query_langs = parse_query_langs(query_languages)
    response = []
    for query_lang in query_langs:
        chars = load_index(query_lang, 'characters')
        response.append({query_lang: list(chars[query_field].keys())})
    return Response(error=False, response=response)


@characters.get('/{query}')
async def query_character(query: str, query_languages: str = 'eng', result_language: str = 'ru') -> Response[Character]:
    query_langs = parse_query_langs(query_languages)
    result_lang = parse_result_lang(result_language)

    filename = ''
    for lang in query_langs:
        index: dict[str, str] = load_index(lang, 'characters')['names']
        for key, value in index.items():
            if key.lower().startswith(query):
                filename = value
                break
        else:
            raise HTTPException(404)
    return Response[Character](response=load_file(result_lang, 'characters', filename))
